from abc import ABC, abstractmethod

class Strategy(ABC):
    """
    The Strategy interface declares operations common to all supported versions
    of some algorithm.

    The Context uses this interface to call the algorithm defined by Concrete
    Strategies.
    """

    @abstractmethod
    def publicacion(self, archivo: str):
        pass







from ..abstract import Strategy
import dropbox, os

class DropboxApi(Strategy):

    def __init__(self, token, path) -> None:
        self.token = token
        self.path = path

    def publicacion(self, archivo: str) -> str:
        file_basename = os.path.basename(archivo)
        client = dropbox.Dropbox(self.token, scope=['files.content.write'])
        client.files_upload(open(archivo, "rb").read(), os.path.join(self.path, file_basename), mode=dropbox.files.WriteMode.overwrite)
        return "[UPLOADED] {}".format(archivo)



from __future__ import print_function

import os
from  ..abstract import Strategy
from cd_drive import Drive
import mimetypes

class GoogleDrive(Strategy):

    def __init__(self, credentials_file, path) -> None:
        self.credentials_file = credentials_file
        self.path = path

    def publicacion(self, archivo: str) -> str:
        mimetypes.init()
        mimetype = mimetypes.guess_type(archivo)[0] or 'text/plain'
        filename = os.path.basename(archivo)

        drive = Drive(self.credentials_file)
        drive.cd(self.path)
        drive.write(filename, mimetype, archivo)



from __future__ import annotations
from .abstract import Strategy

class Publicador():
    """
    The Context defines the interface of interest to clients.
    """

    def __init__(self, archivo, strategy: Strategy) -> None:
        """
        Usually, the Context accepts a strategy through the constructor, but
        also provides a setter to change it at runtime.
        """

        self._strategy = strategy
        self._archivo = archivo

    @property
    def strategy(self) -> Strategy:
        """
        The Context maintains a reference to one of the Strategy objects. The
        Context does not know the concrete class of a strategy. It should work
        with all strategies via the Strategy interface.
        """

        return self._strategy

    @strategy.setter
    def strategy(self, strategy: Strategy) -> None:
        """
        Usually, the Context allows replacing a Strategy object at runtime.
        """

        self._strategy = strategy

    def publicar(self) -> None:
        """
        The Context delegates some work to the Strategy object instead of
        implementing multiple versions of the algorithm on its own.
        """

        result = self._strategy.publicacion(self._archivo)



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="publicador",
    version="0.0.1",
    author="José Luis Di Biase",
    author_email="josx@camba.coop",
    description="Publicador de archivos en diferentes plataformas",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Cambalab/publicador",
    packages=setuptools.find_packages(),
    install_requires=[
        'dropbox',
        'cd-drive'
    ],
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
        "Topic :: Utilities",
    ],
    python_requires='>=3.6',
)
